@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -18,19 +18,16 @@
 from __future__ import division, print_function, absolute_import
 
 import math
-from collections import namedtuple
 
 import hypothesis.internal.conjecture.utils as d
 import hypothesis.internal.conjecture.floats as flt
 from hypothesis.control import assume
-from hypothesis.internal.compat import int_from_bytes
 from hypothesis.internal.floats import sign
 from hypothesis.searchstrategy.strategies import SearchStrategy, \
     MappedSearchStrategy
 
 
 class IntStrategy(SearchStrategy):
-
     """A generic strategy for integer types that provides the basic methods
     other than produce.
 
@@ -56,24 +53,26 @@ class IntegersFromStrategy(SearchStrategy):
 
 class WideRangeIntStrategy(IntStrategy):
 
+    distribution = d.Sampler([
+        4.0, 8.0, 1.0, 1.0, 0.5
+    ])
+
+    sizes = [8, 16, 32, 64, 128]
+
     def __repr__(self):
         return 'WideRangeIntStrategy()'
 
     def do_draw(self, data):
-        size = 16
-        sign_mask = 2 ** (size * 8 - 1)
-
-        byt = data.draw_bytes(size)
-        r = int_from_bytes(byt)
-        negative = r & sign_mask
-        r &= (~sign_mask)
-        if negative:
+        size = self.sizes[self.distribution.sample(data)]
+        r = data.draw_bits(size)
+        sign = r & 1
+        r >>= 1
+        if sign:
             r = -r
         return int(r)
 
 
 class BoundedIntStrategy(SearchStrategy):
-
     """A strategy for providing integers in some interval with inclusive
     endpoints."""
 
@@ -101,7 +100,6 @@ NASTY_FLOATS.extend([-x for x in NASTY_FLOATS])
 
 
 class FloatStrategy(SearchStrategy):
-
     """Generic superclass for strategies which produce floats."""
 
     def __init__(self, allow_infinity, allow_nan):
@@ -113,8 +111,8 @@ class FloatStrategy(SearchStrategy):
 
         self.nasty_floats = [f for f in NASTY_FLOATS if self.permitted(f)]
         weights = [
-            0.6 * len(self.nasty_floats)
-        ] + [0.4] * len(self.nasty_floats)
+            0.2 * len(self.nasty_floats)
+        ] + [0.8] * len(self.nasty_floats)
         self.sampler = d.Sampler(weights)
 
     def __repr__(self):
@@ -147,17 +145,12 @@ def float_order_key(k):
 
 
 class FixedBoundedFloatStrategy(SearchStrategy):
-
     """A strategy for floats distributed between two endpoints.
 
     The conditional distribution tries to produce values clustered
     closer to one of the ends.
 
     """
-    Parameter = namedtuple(
-        'Parameter',
-        ('cut', 'leftwards')
-    )
 
     def __init__(self, lower_bound, upper_bound):
         SearchStrategy.__init__(self)
@@ -192,7 +185,6 @@ class FixedBoundedFloatStrategy(SearchStrategy):
 
 
 class ComplexStrategy(MappedSearchStrategy):
-
     """A strategy over complex numbers, with real and imaginary values
     distributed according to some provided strategy for floating point
     numbers."""
