@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2017 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -30,15 +30,16 @@ from __future__ import division, print_function, absolute_import
 import inspect
 import traceback
 from unittest import TestCase
-from collections import namedtuple
+
+import attr
 
 import hypothesis.internal.conjecture.utils as cu
 from hypothesis.core import find
 from hypothesis.errors import Flaky, NoSuchExample, InvalidDefinition, \
     HypothesisException
 from hypothesis.control import BuildContext
-from hypothesis._settings import settings as Settings
 from hypothesis._settings import Verbosity
+from hypothesis._settings import settings as Settings
 from hypothesis.reporting import report, verbose_report, current_verbosity
 from hypothesis.strategies import just, lists, builds, one_of, runner, \
     integers
@@ -115,7 +116,6 @@ def run_state_machine_as_test(state_machine_factory, settings=None):
 
 
 class GenericStateMachine(object):
-
     """A GenericStateMachine is the basic entry point into Hypothesis's
     approach to stateful testing.
 
@@ -210,7 +210,6 @@ GenericStateMachine.find_breaking_runner = classmethod(find_breaking_runner)
 
 
 class StateMachineRunner(object):
-
     """A StateMachineRunner is a description of how to run a state machine.
 
     It contains values that it will use to shape the examples.
@@ -260,10 +259,13 @@ class StateMachineSearchStrategy(SearchStrategy):
         return StateMachineRunner(data, self.program_size)
 
 
-Rule = namedtuple(
-    u'Rule',
-    (u'targets', u'function', u'arguments', u'precondition')
-)
+@attr.s()
+class Rule(object):
+    targets = attr.ib()
+    function = attr.ib()
+    arguments = attr.ib()
+    precondition = attr.ib()
+
 
 self_strategy = runner()
 
@@ -330,7 +332,9 @@ def rule(targets=(), target=None, **kwargs):
     return accept
 
 
-VarReference = namedtuple(u'VarReference', (u'name',))
+@attr.s()
+class VarReference(object):
+    name = attr.ib()
 
 
 def precondition(precond):
@@ -377,10 +381,10 @@ def precondition(precond):
     return decorator
 
 
-Invariant = namedtuple(
-    'Invariant',
-    ('function', 'precondition')
-)
+@attr.s()
+class Invariant(object):
+    function = attr.ib()
+    precondition = attr.ib()
 
 
 def invariant():
@@ -417,11 +421,13 @@ def invariant():
     return accept
 
 
-ShuffleBundle = namedtuple('ShuffleBundle', ('bundle', 'swaps'))
+@attr.s()
+class ShuffleBundle(object):
+    bundle = attr.ib()
+    swaps = attr.ib()
 
 
 class RuleBasedStateMachine(GenericStateMachine):
-
     """A RuleBasedStateMachine gives you a more structured way to define state
     machines.
 
@@ -432,6 +438,7 @@ class RuleBasedStateMachine(GenericStateMachine):
     executed.
 
     """
+
     _rules_per_class = {}
     _invariants_per_class = {}
     _base_rules_per_class = {}
